@@ -6,12 +6,12 @@
  *  tunnel endpoints.  It has less protocol overhead than IPSec in Tunnel
  *  mode and allows tunneling of every ETHER TYPE protocol (e.g.
  *  ethernet, ip, arp ...). satp directly includes cryptography and
- *  message authentication based on the methodes used by SRTP.  It is
+ *  message authentication based on the methods used by SRTP.  It is
  *  intended to deliver a generic, scaleable and secure solution for
  *  tunneling and relaying of packets of any protocol.
  *
  *
- *  Copyright (C) 2007-2009 Othmar Gsenger, Erwin Nindl,
+ *  Copyright (C) 2007-2014 Markus Grüneis, Othmar Gsenger, Erwin Nindl,
  *                          Christian Pointner <satp@wirdorange.org>
  *
  *  This file is part of Anytun.
@@ -27,8 +27,22 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with anytun.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Anytun.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *  In addition, as a special exception, the copyright holders give
+ *  permission to link the code of portions of this program with the
+ *  OpenSSL library under certain conditions as described in each
+ *  individual source file, and distribute linked combinations
+ *  including the two.
+ *  You must obey the GNU General Public License in all respects
+ *  for all of the code used other than OpenSSL.  If you modify
+ *  file(s) with this exception, you may extend this exception to your
+ *  version of the file(s), but you are not obligated to do so.  If you
+ *  do not wish to do so, delete this exception statement from your
+ *  version.  If you delete this exception statement from all source
+ *  files in the program, then also delete it here.
  */
+
 #pragma once
 #ifndef ANYTUN_sysexec_hpp_INCLUDED
 #define ANYTUN_sysexec_hpp_INCLUDED
@@ -69,7 +83,7 @@ bool endsWith(std::string const& string, std::string const& suffix)
   return string.find(suffix, string.size() - suffix.size()) != std::string::npos;
 }
 
-void SysExec::doExec(StringVector args, StringList env)
+void SysExec::doExec(StringVector args, StringList env_param)
 {
   std::vector<char> arguments;
 
@@ -105,7 +119,7 @@ void SysExec::doExec(StringVector args, StringList env)
   STARTUPINFOA startup_info = getStartupInfo();
 
   std::map<std::string, std::string> envDict;
-  for(StringList::const_iterator it = env.begin(); it != env.begin(); ++it) {
+  for(StringList::const_iterator it = env_param.begin(); it != env_param.begin(); ++it) {
     size_t delimiter_pos = it->find('=');
     envDict.insert(std::make_pair(it->substr(0, delimiter_pos), it->substr(delimiter_pos + 1)));
   }
@@ -121,7 +135,7 @@ void SysExec::doExec(StringVector args, StringList env)
                      NULL,
                      NULL,
                      false,
-                     NULL,
+                     0,
                      &env[0],
                      NULL,
                      &startup_info,
